//
//  MPAppDelegate.m
//  MoviePlayer
//
//  Created by Dominic Rodemer on 07/03/2017.
//  Copyright © 2017 Dominic Rodemer. All rights reserved.
//

#import "MPAppDelegate.h"

#import <BGFoundation/BGFoundation.h>
#import <Sparkle/Sparkle.h>

#import "VLCMedia+MoviePlayer.h"
#import "MPPlayerWindowController.h"
#import "MPLicenseWindowController.h"
#import "MPLicense.h"

@interface MPAppDelegate ()

- (void)presentLicenseWindow;

- (void)processQueuedFilePaths;
- (BOOL)openFilesAtPaths:(NSArray *)filePaths;

@end

@implementation MPAppDelegate

@synthesize playerWindowController;

- (id)init
{
    if (self = [super init])
    {
        queuedFilePaths = [NSMutableArray array];
    }
    
    return self;
}

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    [[NSAppleEventManager sharedAppleEventManager] setEventHandler:self
                                                       andSelector:@selector(handleGetURLEvent:withReplyEvent:)
                                                     forEventClass:kInternetEventClass
                                                        andEventID:kAEGetURL];
    
    if (![[MPLicense currentLicense] isRegistered])
    {
        [self presentLicenseWindow];
    }
    else
    {
        [[SUUpdater sharedUpdater] checkForUpdatesInBackground];
    }
}

- (void)applicationWillTerminate:(NSNotification *)aNotification
{
    
}

- (BOOL)application:(NSApplication *)theApplication openFile:(NSString *)filename
{
    return [self openFilesAtPaths:@[filename]];
}

- (void)application:(NSApplication *)sender openFiles:(NSArray<NSString *> *)filenames
{
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(processQueuedFilePaths) object:nil];
    [queuedFilePaths addObjectsFromArray:filenames];
    [self performSelector:@selector(processQueuedFilePaths) withObject:nil afterDelay:0.5];
    
    [[NSApplication sharedApplication] replyToOpenOrPrint:NSApplicationDelegateReplySuccess];
}



#pragma mark ---
#pragma mark Accessors
#pragma mark ---
- (MPLicenseWindowController *)licenseWindowController
{
    if (!licenseWindowController)
    {
        licenseWindowController = [[MPLicenseWindowController alloc] init];
    }
    
    return licenseWindowController;
}




#pragma mark ---
#pragma mark Actions
#pragma mark ---
- (IBAction)openFile:(id)sender
{
    NSOpenPanel *openPanel = [[NSOpenPanel alloc] init];
    openPanel.canChooseDirectories = NO;
    openPanel.canCreateDirectories = NO;
    openPanel.allowsMultipleSelection = YES;
    openPanel.allowedFileTypes = [VLCMedia mp_supportedFileTypes];
    
    if ([openPanel runModal] == NSModalResponseOK)
    {
        NSArray *fileURLs = [openPanel URLs];
        NSMutableArray *filePaths = [NSMutableArray array];
        for (NSURL *fileURL in fileURLs)
        {
            NSString *filePath = [fileURL path];
            if (filePath)
                [filePaths addObject:filePath];
        }
        
        [self openFilesAtPaths:filePaths];
    }
}

- (IBAction)registerAction:(id)sender
{
    [self presentLicenseWindow];
}



#pragma mark ---
#pragma mark Private
#pragma mark ---
- (void)presentLicenseWindow
{
    [self.licenseWindowController showWindow:self];
}

- (void)processQueuedFilePaths
{
    if (processingQueuedFilePaths)
        return;
    
    processingQueuedFilePaths = YES;
    
    [queuedFilePaths sortUsingSelector:@selector(localizedStandardCompare:)];
    [self openFilesAtPaths:queuedFilePaths];
    [queuedFilePaths removeAllObjects];
    
    processingQueuedFilePaths = NO;
}

- (BOOL)openFilesAtPaths:(NSArray *)filePaths
{
    if (![[MPLicense currentLicense] isRegistered])
    {
        if (![[MPLicense currentLicense] isTrialPeriod])
        {
            [self presentLicenseWindow];
            return NO;
        }
    }
    
    NSMutableArray *mediaItems = [NSMutableArray array];
    for (NSString *filePath in filePaths)
    {
        NSString *extension = [[filePath pathExtension] lowercaseString];
        if ([[VLCMedia mp_supportedMediaFileTypes] containsObject:extension])
        {
            VLCMedia *mediaItem = [[VLCMedia alloc] initWithPath:filePath];
            if (mediaItem)
                [mediaItems addObject:mediaItem];
        }
    }
    
    //Reverse media items: VLCMediaList has weird sorting behaviour...
    NSArray *reverseMediaItems = [[mediaItems reverseObjectEnumerator] allObjects];
    if ([reverseMediaItems count] > 0)
    {
        if (!playerWindowController)
            playerWindowController = [[MPPlayerWindowController alloc] initWithMediaItems:reverseMediaItems];
        else
            [playerWindowController addMediaItems:reverseMediaItems autoplayItem:[reverseMediaItems lastObject]];
        
        [playerWindowController showWindow:self];
    }
    
    return YES;
}



#pragma mark ---
#pragma mark NSAppleEventManager Handler
#pragma mark ---
- (void)handleGetURLEvent:(NSAppleEventDescriptor *)event withReplyEvent:(NSAppleEventDescriptor *)reply
{
    NSURL *url = [NSURL URLWithString:[[event paramDescriptorForKeyword:keyDirectObject] stringValue]];
    
    NSString *query = [url query];
    NSArray *queryPairs = [query componentsSeparatedByString:@"&"];
    NSMutableDictionary *pairs = [NSMutableDictionary dictionary];
    for (NSString *queryPair in queryPairs)
    {
        NSArray *bits = [queryPair componentsSeparatedByString:@"="];
        if ([bits count] < 2)
            continue;
        
        NSString *key = [[bits objectAtIndex:0] stringByReplacingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
        NSString *value = [[bits objectAtIndex:1] stringByReplacingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
        
        [pairs setObject:value forKey:key];
    }
    
    if ([[url host] isEqualToString:@"license"])
    {
        NSString *name =  [pairs bg_stringForKey:@"name"];
        NSString *licenseKey = [pairs bg_stringForKey:@"licenseKey"];
        
        [[MPLicense currentLicense] registerLicenseCode:licenseKey registeredTo:name];
    }
}


@end
