//
//  MPLicense.m
//  MoviePlayer
//
//  Created by Dominic Rodemer on 09.10.17.
//  Copyright © 2017 Dominic Rodemer. All rights reserved.
//

#import "MPLicense.h"

#import <BGFoundation/BGFoundation.h>
#import "CFobLicVerifier.h"

NSString *const MPLicenseDidRegisterNotification = @"MPLicenseDidRegisterNotification";

#define kMPLicenseTrialDuration 7

@interface MPLicense ()

+ (NSString *)productName;
+ (NSString *)publicKey;

- (BOOL)verifyLicenseCode:(NSString *)licenseCode registeredTo:(NSString *)name;

@end

@implementation MPLicense

- (id)init
{
    if (self = [super init])
    {
        verifier = [[CFobLicVerifier alloc] init];
        [verifier setPublicKey:[MPLicense publicKey] error:nil];
        
        if (![[NSUserDefaults standardUserDefaults] objectForKey:@"kMPLicenseTrialStart"])
        {
            [[NSUserDefaults standardUserDefaults] setObject:[NSDate date] forKey:@"kMPLicenseTrialStart"];
            [[NSUserDefaults standardUserDefaults] synchronize];
        }
    }
    
    return self;
}

+ (MPLicense *)currentLicense
{
    static MPLicense *currentLicense = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        currentLicense = [[MPLicense alloc] init];
    });
    
    return currentLicense;
}



#pragma mark ---
#pragma mark Public
#pragma mark ---
- (BOOL)isRegistered
{
    return [self verifyLicenseCode:[self registeredLicenseCode]
                      registeredTo:[self registeredLicenseName]];
}

- (BOOL)isTrialPeriod
{
    return [self remainingDaysInTrialPeriod] > 0;
}

- (NSUInteger)remainingDaysInTrialPeriod
{
    NSDate *trialStartDate = [[NSUserDefaults standardUserDefaults] objectForKey:@"kMPLicenseTrialStart"];
/*
#if DEBUG
    trialStartDate = [NSDate dateWithTimeIntervalSinceNow:-4*(60*60*24)];
#endif
 */
    NSDate *date = [NSDate date];
    
    NSCalendar *calendar = [[NSCalendar alloc] initWithCalendarIdentifier:NSCalendarIdentifierGregorian];
    NSInteger startDay = [calendar ordinalityOfUnit:NSCalendarUnitDay
                                             inUnit:NSCalendarUnitEra
                                            forDate:trialStartDate];
    NSInteger curentDay = [calendar ordinalityOfUnit:NSCalendarUnitDay
                                              inUnit:NSCalendarUnitEra
                                             forDate:date];
    NSInteger days = curentDay - startDay;
    
    NSInteger remainingDays = kMPLicenseTrialDuration - days;
    if (remainingDays < 0)
        return 0;
    if (remainingDays > kMPLicenseTrialDuration)
        return kMPLicenseTrialDuration;
    
    return (NSUInteger)remainingDays;
}

- (BOOL)registerLicenseCode:(NSString *)licenseCode registeredTo:(NSString *)name
{
    BOOL registered = [self verifyLicenseCode:licenseCode registeredTo:name];
    
    if (registered)
    {
        [[NSUserDefaults standardUserDefaults] setObject:name forKey:@"kMPLicenseName"];
        [[NSUserDefaults standardUserDefaults] setObject:licenseCode forKey:@"kMPLicenseLicenseCode"];
        [[NSUserDefaults standardUserDefaults] synchronize];
        
        [[NSNotificationCenter defaultCenter] postNotificationName:MPLicenseDidRegisterNotification
                                                            object:self];
    }
                       
    return registered;
}

- (NSString *)registeredLicenseName
{
    return [[NSUserDefaults standardUserDefaults] objectForKey:@"kMPLicenseName"];
}

- (NSString *)registeredLicenseCode
{
    return [[NSUserDefaults standardUserDefaults] objectForKey:@"kMPLicenseLicenseCode"];
}



#pragma mark ---
#pragma mark Private
#pragma mark ---
+ (NSString *)productName
{
    return @"Movie Player for Mac";
}

+ (NSString *)publicKey
{
    NSArray *publicKey = @[@"-----BEGIN PUBLIC KEY-----\n",
                           @"MIHwMIGoBgcqhkjOOAQB",
                           @"MIGcAkEA9I",
                           @"lOKaU6gfFu6796X63I/rO",
                           @"lwU9Q5nIh4eoG",
                           @"Yc1s20fRG9ev02",
                           @"Y69Qw7pC1MJY3E9Sax2hk",
                           @"jMLYqtxt9wW7L1QIV",
                           @"APS2tigiNaTb",
                           @"M4Xae18YiadxNC",
                           @"JbAkBFjPI841m9bVuEtQI/9oER",
                           @"xkox7Mjq9KqbVTKn",
                           @"UOHQFTng",
                           @"lJN3hkaP5JBCA1x1aus",
                           @"AnrkEp9D8PY46p5ceEL",
                           @"7oA0MAAkAJR",
                           @"NThx6dsuNRMbAyF",
                           @"0PdqxmutTxCGsDKphYt",
                           @"8cott/7qz9mPKoBJqqXzge",
                           @"ML8ngIipXtJl9Pg",
                           @"V9zYTBqASrlS",
                           @"-----END PUBLIC KEY-----\n"];
    
    
    return [publicKey componentsJoinedByString:@""];
}

- (BOOL)verifyLicenseCode:(NSString *)licenseCode registeredTo:(NSString *)name
{
    if ([licenseCode length] == 0)
        return NO;
    if ([name length] == 0)
        return NO;
    
    NSError *error = nil;
    BOOL verified = [verifier verifyRegCode:licenseCode
                                    forName:[NSString stringWithFormat:@"%@,%@", [MPLicense productName], name]
                                      error:&error];
    BGErrorLog(error);
    
    return verified;
}

@end
