//
//  MPPlayerWindowController.m
//  MoviePlayer
//
//  Created by Dominic Rodemer on 09/03/2017.
//  Copyright © 2017 Dominic Rodemer. All rights reserved.
//

#import "MPPlayerWindowController.h"

#import <BGFoundation/BGFoundation.h>
#import <BGDataBinding/BGDataBinding.h>

#import "MPMainMenu.h"
#import "VLCMedia+MoviePlayer.h"

@interface MPPlayerWindowController ()

- (void)resetHideControllsTimer;
- (BOOL)validateUserInterfaceAction:(SEL)action withObject:(id)object fallbackValue:(BOOL)fallbackValue;

- (void)updateSubtitleTracksWithDelay:(NSTimeInterval)delay;
- (void)updateVideoTracksWithDelay:(NSTimeInterval)delay;
- (void)updateAudioTracksWithDelay:(NSTimeInterval)delay;

@end


@implementation MPPlayerWindowController

@dynamic window;

@synthesize mediaList;

@synthesize videoContainerView;
@synthesize videoView;
@synthesize playerControlView;

- (id)initWithMediaItems:(NSArray <VLCMedia *>*)mediaItems
{
    if (self = [super initWithWindowNibName:@"MPPlayerWindowController" owner:self])
    {
        mediaList = [[VLCMediaList alloc] initWithArray:mediaItems];
        
        //https://wiki.videolan.org/VLC_command-line_help/
        NSArray *options = @[@"--no-play-and-pause", // We want every movie to stopping at eof instead of pause
                             @"--no-color", // Don't use color in output (Xcode doesn't show it)
                             @"--no-video-title-show", // Don't show the title on overlay when starting to play
                             @"--verbose=4", // Let's not wreck the logs
                             @"--no-sout-keep",
                             @"--vout=macosx", // Select Mac OS X video output
                             @"--text-renderer=freetype",
                             @"--extraintf=macosx_dialog_provider"]; // Some extra dialog (login, progress) may come up from here
        listPlayer = [[VLCMediaListPlayer alloc] initWithOptions:options];
        listPlayer.mediaPlayer.delegate = self;
        listPlayer.mediaList = mediaList;
        [listPlayer.mediaPlayer bg_addTarget:self
                                      action:@selector(bindMediaPlayerMedia:)
                            forKeyPathChange:BGKeyPath(VLCMediaPlayer, media)];
        
        playlistViewController = [[MPPlaylistViewController alloc] initWithMediaList:mediaList andListPlyer:listPlayer];
        playlistViewController.delegate = self;
        [playlistViewController bg_addTarget:self
                                      action:@selector(bindPlaylistViewControllerVisible:)
                            forKeyPathChange:BGKeyPath(MPPlaylistViewController, visible)];
        
        videoAdjustmentsViewController = [[MPVideoAdjustmentsViewController alloc] initWithMediaPlayer:listPlayer.mediaPlayer];
        videoAdjustmentsViewController.delegate = self;
        [videoAdjustmentsViewController bg_addTarget:self
                                              action:@selector(bindVideoAdjustmentsViewControllerVisible:)
                                    forKeyPathChange:BGKeyPath(MPVideoAdjustmentsViewController, visible)];
        
        equalizerViewController = [[MPEqualizerViewController alloc] initWithMediaPlayer:listPlayer.mediaPlayer];
        equalizerViewController.delegate = self;
        [equalizerViewController bg_addTarget:self
                                       action:@selector(bindEqualizerViewControllerVisible:)
                             forKeyPathChange:BGKeyPath(MPEqualizerViewController, visible)];
    }
    
    return self;
}

- (void)dealloc
{
    [listPlayer stop];
    
    listPlayer.mediaPlayer.drawable = nil;
    listPlayer.mediaPlayer.delegate = nil;
    listPlayer.mediaPlayer.media = nil;
    listPlayer.mediaList = nil;
    listPlayer = nil;
}

- (void)windowDidLoad
{
    [super windowDidLoad];
        
    __weak __typeof__(self) weakSelf = self;
    [videoContainerView setMouseEnteredHandler:^{
        
        [weakSelf showControlsAnimated:YES];
    }];
    [videoContainerView setMouseExitedHandler:^{
        
        [weakSelf hideControlsAnimated:YES];
    }];
    [videoContainerView setMouseMovedHandler:^{
        
        [weakSelf showControlsAnimated:YES];
        [weakSelf resetHideControllsTimer];
    }];
    
    self.window.restorable = NO;
    self.window.minSize = CGSizeMake(550.0, 350.0);
    self.window.maxSize = [[NSScreen mainScreen] frame].size;
    self.window.titleBarView.appearance = [NSAppearance appearanceNamed:NSAppearanceNameVibrantDark];
    self.window.titleBarView.material = NSVisualEffectMaterialDark;
    self.window.titleBarView.state = NSVisualEffectStateActive;
    self.window.toolbar.showsBaselineSeparator = NO;
    
    [self.window.contentView addSubview:equalizerViewController.view
                             positioned:NSWindowBelow
                             relativeTo:playerControlView];
    [equalizerViewController hideToEdge:MPSidePanelViewControllerEdgeBottom animated:NO];
    
    [self.window.contentView addSubview:videoAdjustmentsViewController.view
                             positioned:NSWindowBelow
                             relativeTo:playerControlView];
    [videoAdjustmentsViewController hideToEdge:MPSidePanelViewControllerEdgeRight animated:NO];
    
    [self.window.contentView addSubview:playlistViewController.view
                             positioned:NSWindowBelow
                             relativeTo:playerControlView];
    [playlistViewController hideToEdge:MPSidePanelViewControllerEdgeRight animated:NO];
    
    [listPlayer.mediaPlayer setDrawable:videoView];
    [listPlayer.mediaPlayer.audio setVolume:playerControlView.volumeSlider.intValue];
    
    [self resetHideControllsTimer];
}

- (IBAction)showWindow:(id)sender
{
    [super showWindow:sender];
    
    [listPlayer play];
    
    [[MPMainMenu mainMenu] setSubtitlesDelayViewEnabled:YES];
}



#pragma mark ---
#pragma mark NSResponder
#pragma mark ---
- (void)mouseDown:(NSEvent *)event
{
    if (playlistViewController.visible)
        [playlistViewController hideToEdge:MPSidePanelViewControllerEdgeRight animated:YES];
    if (videoAdjustmentsViewController.visible)
        [videoAdjustmentsViewController hideToEdge:MPSidePanelViewControllerEdgeRight animated:YES];
    if (equalizerViewController.visible)
        [equalizerViewController hideToEdge:MPSidePanelViewControllerEdgeBottom animated:YES];
}

- (IBAction)addToPlaylistAction:(id)sender
{
    [playlistViewController addFilesToPlaylist];
}



#pragma mark ---
#pragma mark Bindings
#pragma mark ---
- (void)bindMediaPlayerMedia:(NSDictionary *)change
{
    NSString *title = [listPlayer.mediaPlayer.media mp_title];
    if (!title)
        title = [[BGApplicationInformation defaultInformation] name];
    self.window.title = title;
    
    NSString *subtitleFilePath = [listPlayer.mediaPlayer.media mp_correspondingSubtitleFilePathInMediaDirectory];
    if (subtitleFilePath)
    {
        NSURL *subtitleFileURL = [NSURL fileURLWithPath:subtitleFilePath];
        [listPlayer.mediaPlayer addPlaybackSlave:subtitleFileURL
                                            type:VLCMediaPlaybackSlaveTypeSubtitle
                                         enforce:NO];
    }
    
    [self updateSubtitleTracksWithDelay:2.0];
    [self updateVideoTracksWithDelay:2.0];
    [self updateAudioTracksWithDelay:2.0];
}

- (void)bindPlaylistViewControllerVisible:(NSDictionary *)change
{
    if (playlistViewController.visible)
        videoContainerView.trackingEnabled = NO;
    else
        videoContainerView.trackingEnabled = YES;
}

- (void)bindVideoAdjustmentsViewControllerVisible:(NSDictionary *)change
{
    if (videoAdjustmentsViewController.visible)
        videoContainerView.trackingEnabled = NO;
    else
        videoContainerView.trackingEnabled = YES;
}

- (void)bindEqualizerViewControllerVisible:(NSDictionary *)change
{
    if (equalizerViewController.visible)
        videoContainerView.trackingEnabled = NO;
    else
        videoContainerView.trackingEnabled = YES;
}



#pragma mark ---
#pragma mark Actions
#pragma mark ---
- (void)hideControlsTimerAction:(id)sender
{
    [self hideControlsAnimated:YES];
    hideControlsTimer = nil;
}

- (IBAction)volumeSliderAction:(id)sender
{
    [listPlayer.mediaPlayer.audio setVolume:playerControlView.volumeSlider.intValue];
}

- (IBAction)volumeUpAction:(id)sender
{
    [listPlayer.mediaPlayer.audio volumeUp];
    playerControlView.volumeSlider.intValue = listPlayer.mediaPlayer.audio.volume;
}

- (IBAction)volumeDownAction:(id)sender
{
    [listPlayer.mediaPlayer.audio volumeDown];
    playerControlView.volumeSlider.intValue = listPlayer.mediaPlayer.audio.volume;
}

- (IBAction)volumeMuteAction:(id)sender
{
    [listPlayer.mediaPlayer.audio setVolume:0];
    playerControlView.volumeSlider.intValue = 0;
}

- (IBAction)playlistButtonAction:(id)sender
{
    if (playlistViewController.visible)
    {
        [playlistViewController hideToEdge:MPSidePanelViewControllerEdgeRight animated:YES];
    }
    else
    {
        [NSAnimationContext beginGrouping];
        [playlistViewController showFromEdge:MPSidePanelViewControllerEdgeRight animated:YES];
        [self hideControlsAnimated:YES];
        [NSAnimationContext endGrouping];
    }
}

- (IBAction)videoTrackMenuItemAction:(id)sender
{
    NSMenuItem *item = (NSMenuItem *)sender;
    listPlayer.mediaPlayer.currentVideoTrackIndex = (int)item.tag;
    
    for (NSMenuItem *subtitleTrackItem in [[[MPMainMenu mainMenu] videoTrackMenu] itemArray])
    {
        if (subtitleTrackItem == item)
            subtitleTrackItem.state = NSOnState;
        else
            subtitleTrackItem.state = NSOffState;
    }
}

- (IBAction)videoAdjustmentsButtonAction:(id)sender
{
    if (videoAdjustmentsViewController.visible)
    {
        [videoAdjustmentsViewController hideToEdge:MPSidePanelViewControllerEdgeRight animated:YES];
    }
    else
    {
        [NSAnimationContext beginGrouping];
        [videoAdjustmentsViewController showFromEdge:MPSidePanelViewControllerEdgeRight animated:YES];
        [self hideControlsAnimated:YES];
        [NSAnimationContext endGrouping];
    }
}

- (IBAction)audioTrackMenuItemAction:(id)sender
{
    NSMenuItem *item = (NSMenuItem *)sender;
    listPlayer.mediaPlayer.currentAudioTrackIndex = (int)item.tag;
    
    for (NSMenuItem *subtitleTrackItem in [[[MPMainMenu mainMenu] audioTrackMenu] itemArray])
    {
        if (subtitleTrackItem == item)
            subtitleTrackItem.state = NSOnState;
        else
            subtitleTrackItem.state = NSOffState;
    }
}

- (IBAction)equalizerButtonAction:(id)sender
{
    if (equalizerViewController.visible)
    {
        [equalizerViewController hideToEdge:MPSidePanelViewControllerEdgeBottom animated:YES];
    }
    else
    {
        [NSAnimationContext beginGrouping];
        [equalizerViewController showFromEdge:MPSidePanelViewControllerEdgeBottom animated:YES];
        [self hideControlsAnimated:YES];
        [NSAnimationContext endGrouping];
    }
}

- (IBAction)addSubtitlesAction:(id)sender
{
    NSOpenPanel *openPanel = [NSOpenPanel openPanel];
    openPanel.allowedFileTypes = [VLCMedia mp_supportedSubtitleFileTypes];
    openPanel.allowsMultipleSelection = NO;
    openPanel.directoryURL = [NSURL fileURLWithPath:[[listPlayer.mediaPlayer.media mp_filePath] stringByDeletingLastPathComponent]];
    
    if ([openPanel runModal] == NSModalResponseOK)
    {
        NSURL *subtitleFileURL = [openPanel URL];
        
        int success = [listPlayer.mediaPlayer addPlaybackSlave:subtitleFileURL
                                                          type:VLCMediaPlaybackSlaveTypeSubtitle
                                                       enforce:YES];
        if (success >= 0)
        {
            //Delay menu update. Since it takes some time until listPlayer.mediaPlayer.videoSubTitlesNames are loaded
            [self updateSubtitleTracksWithDelay:2.0];
        }
        else
        {
            NSError *error = [NSError errorWithDomain:@"net.domzilla.movieplayermac.error"
                                                 code:0
                                             userInfo:@{NSLocalizedDescriptionKey: NSLocalizedString(@"Unable to open subtitles file",
                                                                                                     @"Error message if a subtitle file couldn't be opened")}];
            [[NSAlert alertWithError:error] runModal];
        }
    }
}

- (IBAction)subtitlesTrackMenuItemAction:(id)sender
{
    NSMenuItem *item = (NSMenuItem *)sender;
    listPlayer.mediaPlayer.currentVideoSubTitleIndex = (int)item.tag;
    
    for (NSMenuItem *subtitleTrackItem in [[[MPMainMenu mainMenu] subtitlesTrackMenu] itemArray])
    {
        if (subtitleTrackItem == item)
            subtitleTrackItem.state = NSOnState;
        else
            subtitleTrackItem.state = NSOffState;
    }
}

- (IBAction)subtitlesDelayAction:(id)sender
{
    listPlayer.mediaPlayer.currentVideoSubTitleDelay = ([[[MPMainMenu mainMenu] subtitlesDelaySlider] doubleValue] * 1000000); //Delay in microseconds
}

- (IBAction)timeSliderAction:(id)sender
{
    VLCTime *newTime = [VLCTime timeWithInt:playerControlView.timeSlider.intValue];
    listPlayer.mediaPlayer.time = newTime;
    
    [playerControlView updateTimeSliderWithCurrentTime:listPlayer.mediaPlayer.time
                                         remainingTime:listPlayer.mediaPlayer.remainingTime];
}

- (IBAction)jumpForwardAction:(id)sender
{
    [listPlayer.mediaPlayer mediumJumpForward];
}

- (IBAction)jumpBackwardAction:(id)sender
{
    [listPlayer.mediaPlayer mediumJumpBackward];
}

- (IBAction)nextAction:(id)sender
{
    [playlistViewController playNextItem];
}

- (IBAction)previousAction:(id)sender
{
    if ([listPlayer.mediaPlayer isPlaying])
    {
        float seconds = (float)listPlayer.mediaPlayer.time.intValue / 1000.0;
        if (seconds >= 3
            || ![playlistViewController hasPreviousItem])
        {
            listPlayer.mediaPlayer.time = [VLCTime timeWithInt:0];
        }
        else
        {
            [playlistViewController playPreviousItem];
        }
    }
    else
    {
        listPlayer.mediaPlayer.time = [VLCTime timeWithInt:0];
        [listPlayer.mediaPlayer play];
    }
}

- (IBAction)togglePlayAction:(id)sender
{
    if ([listPlayer.mediaPlayer isPlaying])
        [listPlayer pause];
    else
        [listPlayer play];
}

- (IBAction)stopAction:(id)sender
{
    [listPlayer stop];
}

- (IBAction)nextPlaylistItemAction:(id)sender
{
    [playlistViewController playNextItem];
}

- (IBAction)previousPlaylistItemAction:(id)sender
{
    [playlistViewController playPreviousItem];
}

- (IBAction)snapshotAction:(id)sender
{
#warning TODO: This is not working, probably due to a missing module (have a look at console output.).
    NSString *temporarySnapshotPath = [[NSFileManager defaultManager] bg_uniqueRandomPathForFileWithExtension:@"png" inPath:NSTemporaryDirectory()];
    [listPlayer.mediaPlayer saveVideoSnapshotAt:temporarySnapshotPath
                                      withWidth:0
                                      andHeight:0];
    
    if ([[NSFileManager defaultManager] fileExistsAtPath:temporarySnapshotPath])
    {
        NSSavePanel *exportPanel = [[NSSavePanel alloc] init];
        exportPanel.allowedFileTypes = @[@"png"];
        exportPanel.nameFieldStringValue = @"Snapshot";
        exportPanel.extensionHidden = NO;
        if ([exportPanel runModal] == NSModalResponseOK)
        {
            NSString *snapshotPath = [[exportPanel URL] path];
            
            NSError *error = nil;
            if (![[NSFileManager defaultManager] bg_moveItemAtPath:temporarySnapshotPath toPath:snapshotPath overwrite:YES error:&error])
            {
                if (error)
                    [[NSAlert alertWithError:error] runModal];
            }
        }
    }
}



#pragma mark ---
#pragma mark Public
#pragma mark ---
- (void)addMediaItems:(NSArray <VLCMedia *>*)mediaItems autoplayItem:(VLCMedia *)autoplayItem
{
    [mediaList lock];
    for (VLCMedia *mediaItem in mediaItems)
    {
        [mediaList addMedia:mediaItem];
    }
    [mediaList unlock];
    
    if (autoplayItem)
    {
        [listPlayer playMedia:autoplayItem];
    }
}

- (void)showControlsAnimated:(BOOL)animated
{
    [NSAnimationContext beginGrouping];
    [self.window showTitleBarAnimated:animated];
    [playerControlView showAnimated:animated];
    [NSAnimationContext endGrouping];
}

- (void)hideControlsAnimated:(BOOL)animated
{    
    [NSCursor setHiddenUntilMouseMoves:YES];
    
    [NSAnimationContext beginGrouping];
    [self.window hideTitleBarAnimated:animated];
    [playerControlView hideAnimated:animated];
    [NSAnimationContext endGrouping];
}



#pragma mark ---
#pragma mark Private
#pragma mark ---
- (void)resetHideControllsTimer
{
    [hideControlsTimer invalidate];
    hideControlsTimer = [NSTimer scheduledTimerWithTimeInterval:5.0
                                                         target:self
                                                       selector:@selector(hideControlsTimerAction:)
                                                       userInfo:nil
                                                        repeats:NO];
}

- (BOOL)validateUserInterfaceAction:(SEL)action withObject:(id)object fallbackValue:(BOOL)fallbackValue
{
    if (action == @selector(volumeUpAction:))
        return YES;
    if (action == @selector(volumeDownAction:))
        return YES;
    if (action == @selector(volumeMuteAction:))
        return YES;
    if (action == @selector(playlistButtonAction:))
        return YES;
    if (action == @selector(videoTrackMenuItemAction:))
        return YES;
    if (action == @selector(videoAdjustmentsButtonAction:))
        return YES;
    if (action == @selector(audioTrackMenuItemAction:))
        return YES;
    if (action == @selector(equalizerButtonAction:))
        return YES;
    if (action == @selector(addSubtitlesAction:))
        return listPlayer.mediaPlayer.media != nil;
    if (action == @selector(subtitlesTrackMenuItemAction:))
        return YES;
    if (action == @selector(subtitlesDelayAction:))
        return listPlayer.mediaPlayer.media != nil;
    if (action == @selector(jumpForwardAction:))
        return listPlayer.mediaPlayer.playing;
    if (action == @selector(jumpBackwardAction:))
        return listPlayer.mediaPlayer.playing;
    if (action == @selector(nextAction:))
        return listPlayer.mediaPlayer.playing;
    if (action == @selector(previousAction:))
        return listPlayer.mediaPlayer.playing;
    if (action == @selector(togglePlayAction:))
        return listPlayer.mediaPlayer.media != nil;
    if (action == @selector(stopAction:))
        return listPlayer.mediaPlayer.playing;
    if (action == @selector(nextPlaylistItemAction:))
        return [playlistViewController hasNextItem];
    if (action == @selector(previousPlaylistItemAction:))
        return [playlistViewController hasPreviousItem];
    if (action == @selector(snapshotAction:))
#warning TODO
        return NO;
    
    return fallbackValue;
}

- (void)updateSubtitleTracksWithDelay:(NSTimeInterval)delay
{
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(delay * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        
        [[[MPMainMenu mainMenu] subtitlesTrackMenu] removeAllItems];
        
        NSArray *subtitleNames = self->listPlayer.mediaPlayer.videoSubTitlesNames;
        if ([subtitleNames count] > 0)
        {
            NSArray *subtitleIndexes = self->listPlayer.mediaPlayer.videoSubTitlesIndexes;
            NSUInteger index = 0;
            for (NSString *subtitleName in subtitleNames)
            {
                NSMenuItem *item = [[NSMenuItem alloc] initWithTitle:subtitleName
                                                              action:@selector(subtitlesTrackMenuItemAction:)
                                                       keyEquivalent:@""];
                item.tag = [[subtitleIndexes objectAtIndex:index] intValue];
                if (item.tag == self->listPlayer.mediaPlayer.currentVideoSubTitleIndex)
                    item.state = NSOnState;
                
                [[[MPMainMenu mainMenu] subtitlesTrackMenu] addItem:item];
                index++;
            }
        }
        else
        {
            [[[MPMainMenu mainMenu] subtitlesTrackMenu] addItemWithTitle:NSLocalizedString(@"None", @"Default value if no subtitle, video- or audio-track is available")
                                                                  action:nil
                                                           keyEquivalent:@""];
        }
    });
}

- (void)updateVideoTracksWithDelay:(NSTimeInterval)delay
{
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(delay * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        
        [[[MPMainMenu mainMenu] videoTrackMenu] removeAllItems];
        
        NSArray *videoTrackNames = self->listPlayer.mediaPlayer.videoTrackNames;
        if ([videoTrackNames count] > 0)
        {
            NSArray *videoTrackIndexes = self->listPlayer.mediaPlayer.videoTrackIndexes;
            NSUInteger index = 0;
            for (NSString *videoTrackName in videoTrackNames)
            {
                NSMenuItem *item = [[NSMenuItem alloc] initWithTitle:videoTrackName
                                                              action:@selector(videoTrackMenuItemAction:)
                                                       keyEquivalent:@""];
                item.tag = [[videoTrackIndexes objectAtIndex:index] intValue];
                if (item.tag == self->listPlayer.mediaPlayer.currentVideoTrackIndex)
                    item.state = NSOnState;
                
                [[[MPMainMenu mainMenu] videoTrackMenu] addItem:item];
                index++;
            }
        }
        else
        {
            [[[MPMainMenu mainMenu] videoTrackMenu] addItemWithTitle:NSLocalizedString(@"None", @"Default value if no subtitle, video- or audio-track is available")
                                                              action:nil
                                                       keyEquivalent:@""];
        }
    });
}

- (void)updateAudioTracksWithDelay:(NSTimeInterval)delay
{
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(delay * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        
        [[[MPMainMenu mainMenu] audioTrackMenu] removeAllItems];
        
        NSArray *audioTrackNames = self->listPlayer.mediaPlayer.audioTrackNames;
        if ([audioTrackNames count] > 0)
        {
            NSArray *audioTrackIndexes = self->listPlayer.mediaPlayer.audioTrackIndexes;
            NSUInteger index = 0;
            for (NSString *audioTrackName in audioTrackNames)
            {
                NSMenuItem *item = [[NSMenuItem alloc] initWithTitle:audioTrackName
                                                              action:@selector(audioTrackMenuItemAction:)
                                                       keyEquivalent:@""];
                item.tag = [[audioTrackIndexes objectAtIndex:index] intValue];
                if (item.tag == self->listPlayer.mediaPlayer.currentAudioTrackIndex)
                    item.state = NSOnState;
                
                [[[MPMainMenu mainMenu] audioTrackMenu] addItem:item];
                index++;
            }
        }
        else
        {
            [[[MPMainMenu mainMenu] audioTrackMenu] addItemWithTitle:NSLocalizedString(@"None", @"Default value if no subtitle, video- or audio-track is available")
                                                              action:nil
                                                       keyEquivalent:@""];
        }
    });
}



#pragma mark ---
#pragma mark NSMenuValidation
#pragma mark ---
- (BOOL)validateMenuItem:(NSMenuItem *)menuItem
{
    return [self validateUserInterfaceAction:[menuItem action] withObject:nil fallbackValue:menuItem.enabled];
}



#pragma mark ---
#pragma mark NSWindowDelegate
#pragma mark ---
- (void)windowWillClose:(NSNotification *)notification
{
    if ([listPlayer.mediaPlayer isPlaying])
        [listPlayer stop];
    
    listPlayer.mediaPlayer.media = nil;
    
    [mediaList lock];
    for (NSInteger i = 0; i < [mediaList count]; i++)
    {
        [mediaList removeMediaAtIndex:i];
    }
    [mediaList unlock];
    
    [[MPMainMenu mainMenu] setSubtitlesDelayViewEnabled:NO];
}



#pragma mark ---
#pragma mark VLCMediaPlayerDelegate
#pragma mark ---
- (void)mediaPlayerStateChanged:(NSNotification *)aNotification
{
    CGSize videoSize = listPlayer.mediaPlayer.videoSize;
    if (!CGSizeEqualToSize(nativeVideoSize, videoSize))
    {
        if (!CGSizeEqualToSize(videoSize, CGSizeZero))
        {
            nativeVideoSize = videoSize;
            
            videoContainerView.videoSize = nativeVideoSize;
            [self.window sizeToFitVideoSize:nativeVideoSize
                                aspectRatio:nativeVideoSize
                                   animated:YES];
        }
    }
    
    if (!listPlayer.mediaPlayer.media.length.intValue)
        [playerControlView disableTimeSlider];
    else
        [playerControlView enableTimeSlider];
    
    if ([listPlayer.mediaPlayer isPlaying])
        playerControlView.playButton.image = [NSImage imageNamed:@"button_pause"];
    else
        playerControlView.playButton.image = [NSImage imageNamed:@"button_play"];
    
    switch (listPlayer.mediaPlayer.state)
    {
        case VLCMediaPlayerStatePlaying:
        {
            break;
        }
        case VLCMediaPlayerStatePaused:
        {
            break;
        }
        case VLCMediaPlayerStateEnded:
        {
            if ([playlistViewController hasNextItem])
                [playlistViewController playNextItem];
            
            break;
        }
        case VLCMediaPlayerStateStopped:
        {
            break;
        }
        default:
            break;
    }
}

- (void)mediaPlayerTimeChanged:(NSNotification *)aNotification
{
    int mediaLength = listPlayer.mediaPlayer.media.length.intValue;
    if ((int)playerControlView.timeSlider.maxValue != mediaLength)
    {
        playerControlView.timeSlider.maxValue = mediaLength;
    }
    
    [playerControlView updateTimeSliderWithCurrentTime:listPlayer.mediaPlayer.time
                                         remainingTime:listPlayer.mediaPlayer.remainingTime];
}

- (void)mediaPlayerTitleChanged:(NSNotification *)aNotification
{
    
}

- (void)mediaPlayerChapterChanged:(NSNotification *)aNotification
{
    
}



#pragma mark ---
#pragma mark MPSidePanelViewControllerDelegate
#pragma mark ---
- (void)sidePanelViewControllerWillShow:(MPSidePanelViewController *)controller animated:(BOOL)animated;
{
    [visibleSidePanelViewController hideToEdge:visibleSidePanelViewController.visibleFromEdge animated:animated];
}

- (void)sidePanelViewControllerDidShow:(MPSidePanelViewController *)controller animated:(BOOL)animated;
{
    visibleSidePanelViewController = controller;
}

- (void)sidePanelViewControllerWillHide:(MPSidePanelViewController *)controller animated:(BOOL)animated;
{
    if (visibleSidePanelViewController == controller)
    {
        visibleSidePanelViewController = nil;
    }
}


@end
