//
//  MPPlaylistViewController.m
//  MoviePlayer
//
//  Created by Dominic Rodemer on 27/03/2017.
//  Copyright © 2017 Dominic Rodemer. All rights reserved.
//

#import "MPPlaylistViewController.h"

#import <BGDataBinding/BGDataBinding.h>
#import <BGFoundation/BGFoundation.h>

#import "MPMediaParser.h"
#import "MPPlaylistTableCellView.h"

#import "VLCMedia+MoviePlayer.h"

@interface MPPlaylistViewController ()

- (BOOL)validateUserInterfaceAction:(SEL)action withObject:(id)object fallbackValue:(BOOL)fallbackValue;

@end

@implementation MPPlaylistViewController

@synthesize mediaList;
@synthesize listPlayer;
@synthesize scrollView;
@synthesize tableView;
@synthesize addButton;
@synthesize removeButton;

static NSString *PlaylistCellIdentifier = @"PlaylistCellIdentifier";

- (id)initWithMediaList:(VLCMediaList *)aMediaList andListPlyer:(VLCMediaListPlayer *)aListPlayer
{
    if (self = [super initWithNibName:@"MPPlaylistViewController" bundle:nil])
    {
        mediaList = aMediaList;
        mediaList.delegate = self;
        
        listPlayer = aListPlayer;
        [listPlayer.mediaPlayer bg_addTarget:self
                                      action:@selector(bindMediaPlayerMedia:)
                            forKeyPathChange:BGKeyPath(VLCMediaPlayer, media)];
        
        
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(mediaParserDidParseMediaNotification:)
                                                     name:MPMediaParserDidParseMediaNotification
                                                   object:nil];
    }
    
    return self;
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    scrollView.scrollerStyle = NSScrollerStyleOverlay;
        
    tableView.appearance = [NSAppearance appearanceNamed:NSAppearanceNameVibrantDark];
    [tableView registerNib:[[NSNib alloc] initWithNibNamed:@"MPPlaylistTableCellView" bundle:nil]
             forIdentifier:PlaylistCellIdentifier];
    [tableView registerForDraggedTypes:@[NSFilenamesPboardType, NSFilesPromisePboardType]];
    [tableView setDraggingSourceOperationMask:NSDragOperationCopy forLocal:NO];
}



#pragma mark ---
#pragma mark NSResponder
#pragma mark ---
- (IBAction)delete:(id)sender
{
    NSIndexSet *rows = [tableView selectedRowIndexes];
    
    [tableView beginUpdates];
    [rows enumerateIndexesUsingBlock:^(NSUInteger idx, BOOL * _Nonnull stop) {
        [self->mediaList removeMediaAtIndex:idx];
    }];
    [tableView removeRowsAtIndexes:rows withAnimation:NSTableViewAnimationEffectFade];
    [tableView endUpdates];
}

- (IBAction)duplicate:(id)sender
{
    NSIndexSet *rows = [tableView selectedRowIndexes];
    
    NSMutableIndexSet *insertedRows = [NSMutableIndexSet indexSet];
    [tableView beginUpdates];
    [rows enumerateIndexesUsingBlock:^(NSUInteger idx, BOOL * _Nonnull stop) {
        
        VLCMedia *media = [self->mediaList mediaAtIndex:idx];
        if (media)
        {
            NSString *filePath = [media mp_filePath];
            NSURL *mediaURL = [NSURL fileURLWithPath:filePath];
            if (mediaURL)
            {
                NSUInteger insertionIndex = idx + 1;
                VLCMedia *duplicatedMedia = [VLCMedia mediaWithURL:mediaURL];
                [self->mediaList insertMedia:duplicatedMedia atIndex:insertionIndex];
                [insertedRows addIndex:insertionIndex];
            }
        }
    }];
    [tableView insertRowsAtIndexes:insertedRows withAnimation:NSTableViewAnimationEffectGap];
    [tableView endUpdates];
}

- (IBAction)cancel:(id)sender
{
    [tableView deselectAll:sender];
}



#pragma mark ---
#pragma mark Bindings
#pragma mark ---
- (void)bindMediaPlayerMedia:(NSDictionary *)change
{
    NSMutableIndexSet *affectedRows = [NSMutableIndexSet indexSet];
    
    VLCMedia *previousMedia = [change objectForKey:kBGDataBindingChangeOldKey];
    if (previousMedia)
    {
        NSInteger index = [mediaList indexOfMedia:previousMedia];
        if (index != NSNotFound)
            [affectedRows addIndex:index];
    }
    
    VLCMedia *currentMedia = [change objectForKey:kBGDataBindingChangeNewKey];
    if (currentMedia)
    {
        NSInteger index = [mediaList indexOfMedia:currentMedia];
        if (index != NSNotFound)
            [affectedRows addIndex:index];
    }
    
    [tableView reloadDataForRowIndexes:affectedRows
                         columnIndexes:[NSIndexSet indexSetWithIndex:0]];
}



#pragma mark ---
#pragma mark Actions
#pragma mark ---
- (IBAction)addToPlaylistAction:(id)sender
{
    [self addFilesToPlaylist];
}

- (IBAction)tableViewClickAction:(id)sender
{
    NSInteger row = [tableView clickedRow];
    
    if (row == NSNotFound)
        [tableView deselectAll:sender];
}

- (IBAction)tableViewDoubleClickAction:(id)sender
{
    NSInteger row = [tableView clickedRow];
    
    [listPlayer playItemAtNumber:@(row)];
}



#pragma mark ---
#pragma mark MPSidePanelViewControllerSubclassingHooks
#pragma mark ---
- (void)sidePanelDidShow
{
    [scrollView flashScrollers];
}



#pragma mark ---
#pragma mark Public
#pragma mark ---
- (BOOL)hasNextItem
{
    return ([mediaList indexOfMedia:listPlayer.mediaPlayer.media] < [mediaList count] - 1);
}

- (void)playNextItem
{
    NSInteger index = [mediaList indexOfMedia:listPlayer.mediaPlayer.media];
    if (index < [mediaList count] - 1)
    {
        [listPlayer playItemAtNumber:@(index+1)];
    }
}


- (BOOL)hasPreviousItem
{
    return ([mediaList indexOfMedia:listPlayer.mediaPlayer.media] > 0);
}

- (void)playPreviousItem
{
    NSInteger index = [mediaList indexOfMedia:listPlayer.mediaPlayer.media];
    if (index > 0)
    {
        [listPlayer playItemAtNumber:@(index-1)];
    }
}



#pragma mark ---
#pragma mark Private
#pragma mark ---
- (BOOL)validateUserInterfaceAction:(SEL)action withObject:(id)object fallbackValue:(BOOL)fallbackValue
{
    NSUInteger selectedRows = [tableView.selectedRowIndexes count];
    
    if (action == @selector(delete:))
        return (selectedRows > 0);
    if (action == @selector(duplicate:))
        return (selectedRows > 0);
    if (action == @selector(cancel:))
        return (selectedRows > 0);
    if (action == @selector(addToPlaylistAction:))
        return YES;

    return fallbackValue;
}



#pragma mark ---
#pragma mark Public
#pragma mark ---
- (void)addFilesToPlaylist
{
    NSOpenPanel *openPanel = [[NSOpenPanel alloc] init];
    openPanel.canChooseDirectories = NO;
    openPanel.canCreateDirectories = NO;
    openPanel.allowsMultipleSelection = YES;
    openPanel.allowedFileTypes = [VLCMedia mp_supportedFileTypes];
    openPanel.prompt = NSLocalizedString(@"Add", @"Title for a button to add files");
    
    if ([openPanel runModal] == NSModalResponseOK)
    {
        NSArray *fileURLs = [openPanel URLs];
        
        NSMutableIndexSet *insertedRows = [NSMutableIndexSet indexSet];
        NSUInteger insertionIndex = [mediaList count];
        [tableView beginUpdates];
        for (NSURL *fileURL in fileURLs)
        {
            NSString *extension = [[fileURL pathExtension] lowercaseString];
            if ([[VLCMedia mp_supportedMediaFileTypes] containsObject:extension])
            {
                VLCMedia *media = [VLCMedia mediaWithURL:fileURL];
                if (media)
                {
                    [mediaList insertMedia:media atIndex:insertionIndex];
                    [insertedRows addIndex:insertionIndex];
                    insertionIndex++;
                }
            }
        }
        [tableView insertRowsAtIndexes:insertedRows withAnimation:NSTableViewAnimationEffectGap];
        [tableView endUpdates];
    }
}



#pragma mark ---
#pragma mark NSToolbarItemValidation
#pragma mark ---
- (BOOL)validateToolbarItem:(NSToolbarItem *)toolbarItem
{
    return [self validateUserInterfaceAction:[toolbarItem action] withObject:nil fallbackValue:toolbarItem.enabled];
}



#pragma mark ---
#pragma mark NSMenuValidation
#pragma mark ---
- (BOOL)validateMenuItem:(NSMenuItem *)menuItem
{
    return [self validateUserInterfaceAction:[menuItem action] withObject:nil fallbackValue:menuItem.enabled];
}



#pragma mark ---
#pragma mark NSMenuDelegate
#pragma mark ---
- (void)menuWillOpen:(NSMenu *)menu
{
    NSIndexSet *selectedIndexes = [tableView selectedRowIndexes];
    NSInteger clickedRow = [tableView clickedRow];
    if (clickedRow != -1 && ![selectedIndexes containsIndex:clickedRow])
    {
        [tableView selectRowIndexes:[NSIndexSet indexSetWithIndex:clickedRow] byExtendingSelection:NO];
    }
}



#pragma mark ---
#pragma mark NSTableViewDataSource
#pragma mark ---
- (NSInteger)numberOfRowsInTableView:(NSTableView *)aTableView
{
    NSInteger rows = [mediaList count];
    
    return rows;
}

- (BOOL)tableView:(NSTableView *)aTableView writeRowsWithIndexes:(NSIndexSet *)rowIndexes toPasteboard:(NSPasteboard *)pboard
{
    NSMutableArray *filePaths = [NSMutableArray array];
    [rowIndexes enumerateIndexesUsingBlock:^(NSUInteger row, BOOL *stop) {
        
        VLCMedia *media = [self->mediaList mediaAtIndex:row];
        NSString *filePath = [media mp_filePath];
        
        if (filePath)
            [filePaths addObject:filePath];
    }];
    
    if ([filePaths count] > 0)
    {
        [pboard setPropertyList:filePaths
                        forType:NSFilesPromisePboardType];
        
        return YES;
    }
    
    return NO;
}

- (void)tableView:(NSTableView *)aTableView draggingSession:(NSDraggingSession *)session willBeginAtPoint:(NSPoint)screenPoint forRowIndexes:(NSIndexSet *)rowIndexes
{
    NSUInteger start = [rowIndexes firstIndex];
    NSUInteger lenght = ([rowIndexes lastIndex] + 1) - start;
    draggingRange.location = start;
    draggingRange.length = lenght;
    
    NSMutableArray *mediaItems = [NSMutableArray array];
    [rowIndexes enumerateIndexesUsingBlock:^(NSUInteger row, BOOL *stop) {
        
        VLCMedia *media = [self->mediaList mediaAtIndex:row];
        if (media)
            [mediaItems addObject:media];
    }];
    draggedItems = [NSArray arrayWithArray:mediaItems];
}

- (void)tableView:(NSTableView *)aTableView draggingSession:(NSDraggingSession *)session endedAtPoint:(NSPoint)screenPoint operation:(NSDragOperation)operation
{
    draggingRange.location = 0;
    draggingRange.length = 0;
    
    draggedItems = nil;
}

- (void)tableView:(NSTableView *)aTableView updateDraggingItemsForDrag:(id<NSDraggingInfo>)info
{
    
}

- (NSDragOperation)tableView:(NSTableView *)aTableView validateDrop:(id<NSDraggingInfo>)info proposedRow:(NSInteger)row proposedDropOperation:(NSTableViewDropOperation)operation
{
    id draggingSource = [info draggingSource];
    NSDragOperation draggingOperationMask = [info draggingSourceOperationMask];
    
    if (operation == NSTableViewDropAbove)
    {
        if (draggingOperationMask == NSDragOperationCopy
            || draggingSource != aTableView) //Copy or dragged from other source
        {
            return NSDragOperationCopy;
        }
        
        if (draggingRange.location > row
            || draggingRange.location + draggingRange.length < row) //Move
        {
            return NSDragOperationMove;
        }
    }
    
    return NSDragOperationNone;
}

- (BOOL)tableView:(NSTableView *)aTableView acceptDrop:(id<NSDraggingInfo>)info row:(NSInteger)row dropOperation:(NSTableViewDropOperation)operation
{
    BOOL acceptDrop = NO;
    
    if (operation == NSTableViewDropAbove)
    {
        id draggingSource = [info draggingSource];
        NSDragOperation draggingOperationMask = [info draggingSourceOperationMask];
        
        if (draggingSource == tableView
            && draggedItems != nil
            && draggingOperationMask != NSDragOperationCopy) //Move
        {
            [tableView beginUpdates];
            [draggedItems enumerateObjectsUsingBlock:^(VLCMedia *item, NSUInteger idx, BOOL *stop) {
                
                NSInteger newIndex = row + idx;
                NSUInteger oldIndex = [self->mediaList indexOfMedia:item];
                if (oldIndex < newIndex)
                    newIndex -= idx+1;
                
                [self->mediaList removeMediaAtIndex:oldIndex];
                [self->mediaList insertMedia:item atIndex:newIndex];
                [self->tableView moveRowAtIndex:oldIndex toIndex:newIndex];
            }];
            [tableView endUpdates];
            
            acceptDrop = YES;
        }
        else
        {
            NSMutableArray *importMediaURLs = [NSMutableArray array];
            
            if (draggingSource == nil) //Add from external source
            {
                NSPasteboard *pboard = info.draggingPasteboard;
                NSArray *filesPaths = [pboard propertyListForType:NSFilenamesPboardType];
                
                for (NSString *filePath in filesPaths)
                {
                    NSURL *fileURL = [NSURL fileURLWithPath:filePath];
                    if (fileURL)
                        [importMediaURLs addObject:fileURL];
                }
            }
            else if (draggingSource == tableView) //Copy
            {
                [draggedItems enumerateObjectsUsingBlock:^(VLCMedia *item, NSUInteger idx, BOOL *stop) {
                    
                    NSString *filePath = [item mp_filePath];
                    NSURL *fileURL = [NSURL fileURLWithPath:filePath];
                    if (fileURL)
                        [importMediaURLs addObject:fileURL];
                }];
            }
            
            [tableView beginUpdates];
            NSMutableIndexSet *insertedRows = [NSMutableIndexSet indexSet];
            [importMediaURLs enumerateObjectsUsingBlock:^(NSURL *mediaURL, NSUInteger idx, BOOL *stop) {
                
                NSInteger index = row + idx;
                
                NSString *extension = [[mediaURL pathExtension] lowercaseString];
                if ([[VLCMedia mp_supportedMediaFileTypes] containsObject:extension])
                {
                    VLCMedia *media = [VLCMedia mediaWithURL:mediaURL];
                    if (media)
                    {
                        [self->mediaList insertMedia:media atIndex:index];
                        [insertedRows addIndex:index];
                    }
                }
            }];
            
            if ([insertedRows count] > 0)
            {
                [tableView insertRowsAtIndexes:insertedRows withAnimation:NSTableViewAnimationEffectGap];
                acceptDrop = YES;
            }
            [tableView endUpdates];
        }
    }
    
    return acceptDrop;
}

- (NSArray *)tableView:(NSTableView *)aTableView namesOfPromisedFilesDroppedAtDestination:(NSURL *)dropDestination forDraggedRowsWithIndexes:(NSIndexSet *)indexSet
{
    NSMutableArray *promisedFileNames = [NSMutableArray array];
    
    for (VLCMedia *item in draggedItems)
    {
        NSString *originalFilePath = [item mp_filePath];
        NSString *destinationFilePath = [[NSFileManager defaultManager] bg_uniquePathForFile:[originalFilePath lastPathComponent] inPath:[dropDestination path]];
        
        //Create promised files
        NSError *error = nil;
        if ([[NSFileManager defaultManager] copyItemAtPath:originalFilePath toPath:destinationFilePath error:&error])
        {
            NSString *destinationFileName = [destinationFilePath lastPathComponent];
            [promisedFileNames addObject:destinationFileName];
        }
        BGErrorLog(error);
    }
    
    return promisedFileNames;
}



#pragma mark ---
#pragma mark NSTableViewDelegate
#pragma mark ---
- (NSView *)tableView:(NSTableView *)aTableView viewForTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row
{
    VLCMedia *media = [mediaList mediaAtIndex:row];
    MPPlaylistTableCellView *cell = [tableView makeViewWithIdentifier:PlaylistCellIdentifier owner:self];
    
    cell.textField.stringValue = [media mp_title];
    cell.timeTextField.stringValue = [media.length stringValue];
    cell.imageView.image = [media mp_fileIcon];
    
    if ([mediaList indexOfMedia:listPlayer.mediaPlayer.media] == row)
        cell.accessoryImageView.image = [NSImage imageNamed:@"accessory_play"];
    else
        cell.accessoryImageView.image = nil;
    
    return cell;
}

- (CGFloat)tableView:(NSTableView *)aTableView heightOfRow:(NSInteger)row
{
    return [MPPlaylistTableCellView cellHeight];
}

- (BOOL)tableView:(NSTableView *)aTableView isGroupRow:(NSInteger)row
{
    return NO;
}

- (void)tableViewSelectionDidChange:(NSNotification *)notification
{
    removeButton.enabled = (tableView.numberOfSelectedRows > 0);
}



#pragma mark ---
#pragma mark VLCMediaListDelegate
#pragma mark ---
- (void)mediaList:(VLCMediaList *)aMediaList mediaAdded:(VLCMedia *)media atIndex:(NSInteger)index
{
    [tableView reloadData];
    
    [[MPMediaParser sharedParser] parseMedia:media];
}

- (void)mediaList:(VLCMediaList *)aMediaList mediaRemovedAtIndex:(NSInteger)index
{    
    [tableView reloadData];
}



#pragma mark ---
#pragma mark MPMediaParser Notifications
#pragma mark ---
- (void)mediaParserDidParseMediaNotification:(NSNotification *)notification
{
    VLCMedia *media = [notification.userInfo objectForKey:MPMediaParsereMediaKey];
    
    NSInteger index = [mediaList indexOfMedia:media];
    [tableView reloadDataForRowIndexes:[NSIndexSet indexSetWithIndex:index]
                         columnIndexes:[NSIndexSet indexSetWithIndex:0]];
}

@end
